import io
import string
import base64
import numpy
import csv
import decimal


class QueryData(object):
    def __init__(self, data: list):
        self.data = data

    def fetch_one(self) -> string:
        if len(self.data):
            return self.data[0]
        return []

    def fetch_many(self, size: int) -> list:
        if len(self.data):
            if size > len(self.data):
                return self.data
            else:
                return self.data[0:size]
        return []

    def fetch_all(self):
        return self.data


class Field(object):
    def __init__(self):
        self.name = None
        self.field_type = None
        self.precision = None
        self.scale = None
        self.length = None
        self.nullable = None

    def set_name(self, name):
        self.name = name

    def set_type(self, type):
        self.field_type = type

    def set_precision(self, precision):
        self.precision = precision

    def set_scale(self, scale):
        self.scale = scale

    def set_length(self, length):
        self.length = length

    def set_nullable(self, nullable):
        self.nullable = nullable


class QueryResult(object):
    def __init__(self, total_msg: string, query_type: str):
        self.data = None
        self.state = None
        self.total_row_count = 0
        self.total_msg = total_msg
        self.schema = []
        self.query_type = query_type
        self._parse_result_data()

    def _parse_field(self, field: str, schema_field: Field):
        schema_field.set_name(field['name'])
        if field['type'].__contains__('charTypeInfo'):
            schema_field.set_type(field['type']['category'])
            schema_field.set_nullable(str(field['type']['nullable']) != 'False')
            schema_field.set_length(field['type']['charTypeInfo']['length'])
        elif field['type'].__contains__('decimalTypeInfo'):
            schema_field.set_type(field['type']['category'])
            schema_field.set_nullable(str(field['type']['nullable']) == 'true')
            schema_field.set_precision(field['type']['decimalTypeInfo']['precision'])
            schema_field.set_scale(field['type']['decimalTypeInfo']['scale'])
        else:
            schema_field.set_type(field['type']['category'])
            schema_field.set_nullable(str(field['type']['nullable']) == 'true')

    def get_result_state(self) -> string:
        return self.total_msg['status']['state']

    def _parse_result_data(self):
        if self.query_type == 'anon':
            field = Field()
            field.set_name('RESULT_MESSAGE')
            field.set_type("STRING")
            self.schema.append(field)
            self.total_row_count = 1
            result_data = [['OPERATION SUCCEED']]
            self.data = QueryData(result_data)
            return
        self.state = self.total_msg['status']['state']
        if self.state != 'FAILED':
            if self.query_type == 'create' or self.query_type == 'alter' or self.query_type == 'drop' or self.query_type == 'truncate' or self.query_type == 'insert':
                field = Field()
                field.set_name('RESULT_MESSAGE')
                field.set_type("STRING")
                self.schema.append(field)
                self.total_row_count = 1
                result_data = [['OPERATION SUCCEED']]
                self.data = QueryData(result_data)
            else:
                if not (len(self.total_msg['resultSet']['data']['data'])):
                    self.total_row_count = 0
                    fields = self.total_msg['resultSet']['metadata']['fields']
                    for field in fields:
                        schema_field = Field()
                    self._parse_field(field, schema_field)
                    self.schema.append(schema_field)
                    self.data = QueryData([])
                    return
                result_data = self.total_msg['resultSet']['data']['data']
                data_list = []
                if len(result_data):
                    for singe_data in result_data:
                        decode_data = base64.b64decode(singe_data).strip().decode('utf-8')
                        sub_data_list = decode_data.split('\n')
                        for sub_data_item in sub_data_list:
                            data_list.append(sub_data_item)
                self.total_row_count = len(data_list)
                fields = self.total_msg['resultSet']['metadata']['fields']
                for field in fields:
                    schema_field = Field()
                    self._parse_field(field, schema_field)
                    self.schema.append(schema_field)
                result_data = []
                csv.register_dialect('cz_dialect', delimiter=',', escapechar='\\')
                for row in data_list:
                    data_list = []
                    has_null = False
                    if self.query_type == 'select' or self.query_type == 'desc':
                        if "\\N" in row:
                            has_null = True
                        reader = csv.reader(io.StringIO(row), dialect='cz_dialect')
                        for line in reader:
                            for i in range(len(line)):
                                if self.schema[i].field_type.startswith('DECIMAL') or \
                                        self.schema[i].field_type.startswith('decimal'):
                                    if line[i] == "N" and has_null:
                                        data_list.append(None)
                                    else:
                                        decimal_value = decimal.Decimal(line[i]).to_eng_string()
                                        data_list.append(decimal_value)
                                elif self.schema[i].field_type.startswith('INT') or \
                                        self.schema[i].field_type.startswith('int'):
                                    if line[i] == "N" and has_null:
                                        data_list.append(None)
                                    else:
                                        data_list.append(int(line[i]))
                                elif self.schema[i].field_type.startswith('FLOAT') or \
                                        self.schema[i].field_type.startswith('float'):
                                    if line[i] == "N" and has_null:
                                        data_list.append(None)
                                    else:
                                        data_list.append(float(line[i]))
                                elif self.schema[i].field_type == "STRING" or self.schema[i].field_type.startswith(
                                        "VARCHAR") or \
                                        self.schema[i].field_type.startswith("CHAR") or self.schema[
                                    i].field_type == "string" or \
                                        self.schema[i].field_type.startswith("varchar") or self.schema[
                                    i].field_type.startswith("CHAR") or \
                                        self.schema[i].field_type.startswith("TIMESTAMP"):
                                    if line[i] == "N" and has_null:
                                        data_list.append(None)
                                    else:
                                        data_list.append(line[i])

                                else:
                                    data_list.append(line[i])
                        if len(data_list) > len(self.schema):
                            data_list = data_list[0:len(self.schema)]
                    elif self.query_type == 'show':
                        if len(self.schema) == 1:
                            data_list.append(row)
                        else:
                            for value in row.split(','):
                                data_list.append(value)
                            if len(data_list) > len(self.schema):
                                data_list = data_list[0:len(self.schema)]
                    else:
                        data_list.append(row)
                    result_data.append(data_list)
                self.data = QueryData(result_data)
        else:
            field = Field()
            field.set_name('ERROR_MESSAGE')
            field.set_type("STRING")
            self.schema.append(field)
            self.total_row_count = 1
            result_data = [[self.total_msg['status']['message'].split('\n')[0]]]
            self.data = QueryData(result_data)



"""Connection for ClickZetta DB-API."""

import weakref
import logging
import time

import requests
from requests.adapters import HTTPAdapter
from clickzetta.dbapi import cursor
from clickzetta.dbapi import _helpers
from clickzetta.client import Client

is_token_init = False
current_token = None
token_init_time = 0
https_session = None
user_token_dict = {}
user_token_time_dict = {}
https_session_inited = False


@_helpers.raise_on_closed("Operating on a closed connection.")
class Connection(object):
    def __init__(self, client=None):
        if client is None:
            logging.error('Connection must has a LogParams to log in.')
            raise AssertionError('Connection must has a LogParams to log in.')
        else:
            self._owns_client = True

        self._client = client
        current_time = int(time.time())
        unique_key = self._client.login_params.username + "?" + self._client.login_params.instance_name
        if globals()['user_token_dict'].__contains__(unique_key):
            if current_time - globals()['user_token_time_dict'][unique_key] > 24 * 60 * 60:
                new_token = self._client.log_in_cz(self._client.login_params)
                globals()['user_token_dict'][unique_key] = new_token
                globals()['user_token_time_dict'][unique_key] = current_time
                self._client.token = new_token
            else:
                self._client.token = globals()['user_token_dict'][unique_key]
        else:
            new_token = self._client.log_in_cz(self._client.login_params)
            globals()['user_token_dict'][unique_key] = new_token
            globals()['user_token_time_dict'][unique_key] = current_time
            self._client.token = new_token

        if not globals()['https_session_inited']:
            session = requests.Session()
            session.mount(self._client.base_url, HTTPAdapter(pool_connections=20, pool_maxsize=100, max_retries=1000))
            globals()['https_session'] = session
            globals()['https_session_inited'] = True

        self._client.session = globals()['https_session']
        self._closed = False
        self._cursors_created = weakref.WeakSet()

    def close(self):
        self._closed = True

        if self._owns_client:
            self._client.close()

        for cursor_ in self._cursors_created:
            if not cursor_._closed:
                cursor_.close()

    def commit(self):
        """No-op, but for consistency raise an error if connection is closed."""

    def cursor(self):
        """Return a new cursor object."""

        new_cursor = cursor.Cursor(self)
        self._cursors_created.add(new_cursor)
        return new_cursor


def connect(client=None):
    return Connection(client)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: metadata_entity.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import object_identifier_pb2 as object__identifier__pb2
from . import workspace_meta_pb2 as workspace__meta__pb2
from . import schema_pb2 as schema__pb2
from . import table_common_pb2 as table__common__pb2
from . import table_meta_pb2 as table__meta__pb2
from . import account_pb2 as account__pb2
from . import role_meta_pb2 as role__meta__pb2
from . import job_meta_pb2 as job__meta__pb2
from . import property_pb2 as property__pb2
from . import virtual_cluster_meta_pb2 as virtual__cluster__meta__pb2
from . import file_meta_data_pb2 as file__meta__data__pb2
from . import rm_app_meta_pb2 as rm__app__meta__pb2
from . import virtual_cluster_size_pb2 as virtual__cluster__size__pb2
from . import share_meta_pb2 as share__meta__pb2
from . import function_meta_pb2 as function__meta__pb2
from . import connection_meta_pb2 as connection__meta__pb2
from . import network_policy_pb2 as network__policy__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15metadata_entity.proto\x12\x08\x63z.proto\x1a\x17object_identifier.proto\x1a\x14workspace_meta.proto\x1a\x0cschema.proto\x1a\x12table_common.proto\x1a\x10table_meta.proto\x1a\raccount.proto\x1a\x0frole_meta.proto\x1a\x0ejob_meta.proto\x1a\x0eproperty.proto\x1a\x1avirtual_cluster_meta.proto\x1a\x14\x66ile_meta_data.proto\x1a\x11rm_app_meta.proto\x1a\x1avirtual_cluster_size.proto\x1a\x10share_meta.proto\x1a\x13\x66unction_meta.proto\x1a\x15\x63onnection_meta.proto\x1a\x14network_policy.proto\"\xec\x07\n\x06\x45ntity\x12\x33\n\nidentifier\x18\x01 \x01(\x0b\x32\x1a.cz.proto.ObjectIdentifierH\x01\x88\x01\x01\x12\x0f\n\x07\x63reator\x18\x02 \x01(\x03\x12-\n\x0c\x63reator_type\x18\x03 \x01(\x0e\x32\x17.cz.proto.PrincipalType\x12\x14\n\x07\x63omment\x18\x04 \x01(\tH\x02\x88\x01\x01\x12&\n\nproperties\x18\x05 \x03(\x0b\x32\x12.cz.proto.Property\x12\x13\n\x0b\x63reate_time\x18\x06 \x01(\x03\x12\x18\n\x10last_modify_time\x18\x07 \x01(\x03\x12)\n\x05state\x18\x08 \x01(\x0e\x32\x1a.cz.proto.ObjectState.Type\x12+\n\x08\x63\x61tegory\x18\t \x01(\x0e\x32\x19.cz.proto.Entity.Category\x12(\n\tworkspace\x18\n \x01(\x0b\x32\x13.cz.proto.WorkspaceH\x00\x12\"\n\x06schema\x18\x0b \x01(\x0b\x32\x10.cz.proto.SchemaH\x00\x12$\n\x05table\x18\x0c \x01(\x0b\x32\x13.cz.proto.TableMetaH\x00\x12\x1e\n\x04user\x18\r \x01(\x0b\x32\x0e.cz.proto.UserH\x00\x12\x1e\n\x04role\x18\x0e \x01(\x0b\x32\x0e.cz.proto.RoleH\x00\x12 \n\x03job\x18\x0f \x01(\x0b\x32\x11.cz.proto.JobMetaH\x00\x12\x37\n\x0fvirtual_cluster\x18\x10 \x01(\x0b\x32\x1c.cz.proto.VirtualClusterMetaH\x00\x12&\n\x04\x66ile\x18\x11 \x01(\x0b\x32\x16.cz.proto.FileMetaDataH\x00\x12N\n\x19virtual_cluster_size_spec\x18\x12 \x01(\x0b\x32).com.clickzetta.rm.VirtualClusterSizeSpecH\x00\x12 \n\x05share\x18\x13 \x01(\x0b\x32\x0f.cz.proto.ShareH\x00\x12&\n\x08\x66unction\x18\x14 \x01(\x0b\x32\x12.cz.proto.FunctionH\x00\x12*\n\nconnection\x18\x15 \x01(\x0b\x32\x14.cz.proto.ConnectionH\x00\x12\x31\n\x0enetwork_policy\x18\x16 \x01(\x0b\x32\x17.cz.proto.NetworkPolicyH\x00\x12 \n\x05index\x18\x17 \x01(\x0b\x32\x0f.cz.proto.IndexH\x00\"1\n\x08\x43\x61tegory\x12\x0b\n\x07MANAGED\x10\x00\x12\x0c\n\x08\x45XTERNAL\x10\x01\x12\n\n\x06SHARED\x10\x02\x42\x08\n\x06\x65ntityB\r\n\x0b_identifierB\n\n\x08_comment\"0\n\nEntityList\x12\"\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x10.cz.proto.Entityb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'metadata_entity_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ENTITY._serialized_start=380
  _ENTITY._serialized_end=1384
  _ENTITY_CATEGORY._serialized_start=1298
  _ENTITY_CATEGORY._serialized_end=1347
  _ENTITYLIST._serialized_start=1386
  _ENTITYLIST._serialized_end=1434
# @@protoc_insertion_point(module_scope)



import datetime


Date = datetime.date
Time = datetime.time
Timestamp = datetime.datetime
DateFromTicks = datetime.date.fromtimestamp
TimestampFromTicks = datetime.datetime.fromtimestamp


def Binary(data):
    # Construct a DB-API binary value
    if isinstance(data, int):
        raise TypeError("cannot convert `int` object to binary")

    try:
        return bytes(data)
    except TypeError:
        if isinstance(data, str):
            return data.encode("utf-8")
        else:
            raise


def TimeFromTicks(ticks, tz=None):
    # Construct a DB-API time value from the given ticks value.

    dt = datetime.datetime.fromtimestamp(ticks, tz=tz)
    return dt.timetz()


class _DBAPITypeObject(object):

    def __init__(self, *values):
        self.values = values

    def __eq__(self, other):
        return other in self.values


STRING = _DBAPITypeObject("STRING", "CHAR", "VARCHAR")
BINARY = _DBAPITypeObject("BINARY", "ARRAY", "STRUCT", "MAP")
NUMBER = _DBAPITypeObject(
    "INT8", "INT32", "INT64", "FLOAT32", "FLOAT64", "DECIMAL", "BOOL"
)
DATETIME = _DBAPITypeObject("TIMESTAMP", "DATE")
ROWID = "ROWID"



"""Cursor for ClickZetta DB-API."""

import collections
from collections import abc as collections_abc
import copy
import logging
import re
from datetime import datetime

from clickzetta.enums import *
from clickzetta.table import EmptyRowIterator
from clickzetta.query_result import QueryResult

_LOGGER = logging.getLogger(__name__)

Column = collections.namedtuple(
    "Column",
    [
        "name",
        "type_code",
        "display_size",
        "internal_size",
        "precision",
        "scale",
        "null_ok",
    ],
)


class Cursor(object):
    def __init__(self, connection):
        self.connection = connection
        self.description = None
        self.rowcount = -1
        self._query_result = None
        self._query_data = None
        self._closed = False

    def close(self):
        self._closed = True

    def _set_rowcount(self, query_result):

        self.rowcount = query_result.total_row_count

    def _set_description(self, query_result: QueryResult):
        if query_result.schema is None:
            self.description = None
            return

        self.description = tuple(
            Column(
                name=field.name,
                type_code=field.field_type,
                display_size=None,
                internal_size=field.length,
                precision=field.precision,
                scale=field.scale,
                null_ok=field.nullable,
            )
            for field in query_result.schema
        )

    def execute(self, operation: str, parameters=None):

        self._execute(operation, parameters)

    def _execute(
            self, operation: str, parameters
    ):
        self._query_data = None
        self._query_job = None
        client = self.connection._client
        if not operation.endswith(';'):
            operation = operation + ';'
        operation_upper = operation.upper().strip()

        if operation_upper.startswith('SELECT') or operation_upper.startswith("SET"):
            self._query_result = client.select_table(client.token, operation, parameters)
        elif operation_upper.startswith("DROP"):
            self._query_result = client.drop_table(client.token, operation)
        elif operation_upper.startswith("CREATE"):
            self._query_result = client.create_table(client.token, operation)
        elif operation_upper.startswith('ALTER'):
            self._query_result = client.alter_table(client.token, operation)
        elif operation_upper.startswith("TRUNCATE"):
            self._query_result = client.truncate_table(client.token, operation)
        elif operation_upper.startswith('SHOW'):
            self._query_result = client.show_table(client.token, operation)
        elif operation_upper.startswith('INSERT'):
            self._query_result = client.insert_table(client.token, operation)
        elif operation_upper.startswith('UPDATE'):
            self._query_result = client.update_table(client.token, operation)
        elif operation_upper.startswith('DESC'):
            self._query_result = client.desc_table(client.token, operation)
        else:
            self._query_result = client.execute_other_sql(client.token, operation)
        self._set_rowcount(self._query_result)
        self._query_data = self._query_result.data
        self._set_description(self._query_result)

    def executemany(self, operations, methods):
        print("not supported yet")

    def fetchone(self):
        try:
            return self._query_data.fetch_one()
        except StopIteration:
            return None

    def fetchmany(self, size=None):
        try:
            return self._query_data.fetch_many(size)
        except StopIteration:
            return None

    def fetchall(self):
        return self._query_data.fetch_all()

    def setinputsizes(self, sizes):
        """No-op, but for consistency raise an error if cursor is closed."""

    def setoutputsize(self, size, column=None):
        """No-op, but for consistency raise an error if cursor is closed."""

    def __iter__(self):
        return iter(self._query_data)



"""Shared helper functions for ClickZetta API classes."""

import base64
import datetime
import decimal
import math
import re
import os
from typing import Optional, Union

import packaging.version

UTC = datetime.timezone.utc
_EPOCH = datetime.datetime(1970, 1, 1, tzinfo=datetime.timezone.utc)
_RFC3339_MICROS = "%Y-%m-%dT%H:%M:%S.%fZ"
_RFC3339_NO_FRACTION = "%Y-%m-%dT%H:%M:%S"

_RFC3339_MICROS_NO_ZULU = "%Y-%m-%dT%H:%M:%S.%f"
_TIMEONLY_WO_MICROS = "%H:%M:%S"
_TIMEONLY_W_MICROS = "%H:%M:%S.%f"
_PROJECT_PREFIX_PATTERN = re.compile(
    r"""
    (?P<project_id>\S+\:[^.]+)\.(?P<dataset_id>[^.]+)(?:$|\.(?P<custom_id>[^.]+)$)
""",
    re.VERBOSE,
)

CZ_EMULATOR_HOST = "https://dev-api.zettadecision.com"

_DEFAULT_HOST = "https://dev-api.zettadecision.com"


def _get_click_zetta_host():
    return os.environ.get(CZ_EMULATOR_HOST, _DEFAULT_HOST)


def _not_null(value, field):
    return value is not None or (field is not None and field.mode != "NULLABLE")


def _int_from_json(value, field):
    if _not_null(value, field):
        return int(value)


def _date_from_iso8601_date(value):
    return datetime.datetime.strptime(value, "%Y-%m-%d").date()


def _datetime_from_microseconds(value):
    return _EPOCH + datetime.timedelta(microseconds=value)


def _float_from_json(value, field):
    if _not_null(value, field):
        return float(value)


def _decimal_from_json(value, field):
    if _not_null(value, field):
        return decimal.Decimal(value)


def _bool_from_json(value, field):
    if _not_null(value, field):
        return value.lower() in ["t", "true", "1"]


def _string_from_json(value, _):
    return value


def _to_bytes(value, encoding="ascii"):
    result = value.encode(encoding) if isinstance(value, str) else value
    if isinstance(result, bytes):
        return result
    else:
        raise TypeError("%r could not be converted to bytes" % (value,))


def _bytes_from_json(value, field):
    if _not_null(value, field):
        return base64.standard_b64decode(_to_bytes(value))


def _timestamp_from_json(value, field):
    if _not_null(value, field):
        return _datetime_from_microseconds(int(value))


def _date_from_json(value, field):
    if _not_null(value, field):
        return _date_from_iso8601_date(value)


_CELLDATA_FROM_JSON = {
    "INT8": _int_from_json,
    "INT16": _int_from_json,
    "INT32": _int_from_json,
    "INT64": _int_from_json,
    "FLOAT32": _float_from_json,
    "FLOAT64": _float_from_json,
    "DECIMAL": _decimal_from_json,
    "BOOL": _bool_from_json,
    "STRING": _string_from_json,
    "VARCHAR": _string_from_json,
    "CHAR": _string_from_json,
    "BINARY": _bytes_from_json,
    "TIMESTAMP": _timestamp_from_json,
    "DATE": _date_from_json
}

_QUERY_PARAMS_FROM_JSON = dict(_CELLDATA_FROM_JSON)


def _field_to_index_mapping(schema):
    return {f.name: i for i, f in enumerate(schema)}


def _field_from_json(resource, field):
    converter = _CELLDATA_FROM_JSON.get(field.field_type, lambda value, _: value)
    if field.mode == "REPEATED":
        return [converter(item["v"], field) for item in resource]
    else:
        return converter(resource, field)


def _row_tuple_from_json(row, schema):
    from .schema import _to_schema_fields

    schema = _to_schema_fields(schema)

    row_data = []
    for field, cell in zip(schema, row["f"]):
        row_data.append(_field_from_json(cell["v"], field))
    return tuple(row_data)


def _int_to_json(value):
    if isinstance(value, int):
        value = str(value)
    return value


def _float_to_json(value) -> Union[None, str, float]:
    if value is None:
        return None

    if isinstance(value, str):
        value = float(value)

    return str(value) if (math.isnan(value) or math.isinf(value)) else float(value)


def _decimal_to_json(value):
    if isinstance(value, decimal.Decimal):
        value = str(value)
    return value


def _bool_to_json(value):
    if isinstance(value, bool):
        value = "true" if value else "false"
    return value


def _bytes_to_json(value):
    if isinstance(value, bytes):
        value = base64.standard_b64encode(value).decode("ascii")
    return value


def _timestamp_to_json_parameter(value):
    if isinstance(value, datetime.datetime):
        if value.tzinfo not in (None, UTC):
            # Convert to UTC and remove the time zone info.
            value = value.replace(tzinfo=None) - value.utcoffset()
        value = "%s %s+00:00" % (value.date().isoformat(), value.time().isoformat())
    return value


def _timestamp_to_json_row(value):
    if isinstance(value, datetime.datetime):

        if value.tzinfo is not None:
            value = value.astimezone(UTC)
        value = value.strftime(_RFC3339_MICROS)
    return value


def _datetime_to_json(value):
    if isinstance(value, datetime.datetime):

        if value.tzinfo is not None:
            value = value.astimezone(UTC)
        value = value.strftime(_RFC3339_MICROS_NO_ZULU)
    return value


def _date_to_json(value):
    if isinstance(value, datetime.date):
        value = value.isoformat()
    return value


def _time_to_json(value):
    if isinstance(value, datetime.time):
        value = value.isoformat()
    return value


_SCALAR_VALUE_TO_JSON_ROW = {
    "INT8": _int_to_json,
    "INT16": _int_to_json,
    "INT32": _int_to_json,
    "INT64": _int_to_json,
    "FLOAT32": _float_to_json,
    "FLOAT64": _float_to_json,
    "DECIMAL": _decimal_to_json,
    "BOOL": _bool_to_json,
    "BINARY": _bytes_to_json,
    "TIMESTAMP": _timestamp_to_json_row,
    "DATE": _date_to_json
}

_SCALAR_VALUE_TO_JSON_PARAM = _SCALAR_VALUE_TO_JSON_ROW.copy()
_SCALAR_VALUE_TO_JSON_PARAM["TIMESTAMP"] = _timestamp_to_json_parameter


def _scalar_field_to_json(field, row_value):
    converter = _SCALAR_VALUE_TO_JSON_ROW.get(field.field_type)
    if converter is None:
        return row_value
    return converter(row_value)


def _repeated_field_to_json(field, row_value):
    values = []
    for item in row_value:
        values.append(_single_field_to_json(field, item))
    return values


def _record_field_to_json(fields, row_value):
    isdict = isinstance(row_value, dict)

    if not isdict and len(row_value) != len(fields):
        msg = "The number of row fields ({}) does not match schema length ({}).".format(
            len(row_value), len(fields)
        )
        raise ValueError(msg)

    record = {}

    if isdict:
        processed_fields = set()

    for subindex, subfield in enumerate(fields):
        subname = subfield.name
        subvalue = row_value.get(subname) if isdict else row_value[subindex]

        if subvalue is not None:
            record[subname] = _field_to_json(subfield, subvalue)

        if isdict:
            processed_fields.add(subname)

    if isdict:
        not_processed = set(row_value.keys()) - processed_fields

        for field_name in not_processed:
            value = row_value[field_name]
            if value is not None:
                record[field_name] = str(value)

    return record


def _single_field_to_json(field, row_value):
    if row_value is None:
        return None

    if field.field_type == "RECORD":
        return _record_field_to_json(field.fields, row_value)

    return _scalar_field_to_json(field, row_value)


def _field_to_json(field, row_value):
    if row_value is None:
        return None

    if field.mode == "REPEATED":
        return _repeated_field_to_json(field, row_value)

    return _single_field_to_json(field, row_value)


def _snake_to_camel_case(value):
    words = value.split("_")
    return words[0] + "".join(map(str.capitalize, words[1:]))


def _get_sub_prop(container, keys, default=None):
    if isinstance(keys, str):
        keys = [keys]

    sub_val = container
    for key in keys:
        if key not in sub_val:
            return default
        sub_val = sub_val[key]
    return sub_val


def _set_sub_prop(container, keys, value):
    if isinstance(keys, str):
        keys = [keys]

    sub_val = container
    for key in keys[:-1]:
        if key not in sub_val:
            sub_val[key] = {}
        sub_val = sub_val[key]
    sub_val[keys[-1]] = value


def _del_sub_prop(container, keys):
    sub_val = container
    for key in keys[:-1]:
        if key not in sub_val:
            sub_val[key] = {}
        sub_val = sub_val[key]
    if keys[-1] in sub_val:
        del sub_val[keys[-1]]


def _int_or_none(value):
    if isinstance(value, int):
        return value
    if value is not None:
        return int(value)


def _str_or_none(value):
    if value is not None:
        return str(value)


def _split_id(full_id):
    with_prefix = _PROJECT_PREFIX_PATTERN.match(full_id)
    if with_prefix is None:
        parts = full_id.split(".")
    else:
        parts = with_prefix.groups()
        parts = [part for part in parts if part]
    return parts


def _build_resource_from_properties(obj, filter_fields):
    partial = {}
    for filter_field in filter_fields:
        api_field = obj._PROPERTY_TO_API_FIELD.get(filter_field)
        if api_field is None and filter_field not in obj._properties:
            raise ValueError("No property %s" % filter_field)
        elif api_field is not None:
            partial[api_field] = obj._properties.get(api_field)
        else:
            partial[filter_field] = obj._properties[filter_field]

    return partial


def _verify_job_config_type(job_config, expected_type, param_name="job_config"):
    if not isinstance(job_config, expected_type):
        msg = (
            "Expected an instance of {expected_type} class for the {param_name} parameter, "
            "but received {param_name} = {job_config}"
        )
        raise TypeError(
            msg.format(
                expected_type=expected_type.__name__,
                param_name=param_name,
                job_config=job_config,
            )
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: operator.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import table_meta_pb2 as table__meta__pb2
from . import table_common_pb2 as table__common__pb2
from . import data_type_pb2 as data__type__pb2
from . import expression_pb2 as expression__pb2
from . import file_format_type_pb2 as file__format__type__pb2
from . import file_meta_data_pb2 as file__meta__data__pb2
from . import input_split_pb2 as input__split__pb2
from . import file_system_pb2 as file__system__pb2
from . import virtual_value_info_pb2 as virtual__value__info__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0eoperator.proto\x12\x08\x63z.proto\x1a\x10table_meta.proto\x1a\x12table_common.proto\x1a\x0f\x64\x61ta_type.proto\x1a\x10\x65xpression.proto\x1a\x16\x66ile_format_type.proto\x1a\x14\x66ile_meta_data.proto\x1a\x11input_split.proto\x1a\x11\x66ile_system.proto\x1a\x18virtual_value_info.proto\"2\n\rColumnMapping\x12\x10\n\x08outputId\x18\x01 \x01(\x04\x12\x0f\n\x07inputId\x18\x02 \x01(\x04\"\x8c\x0b\n\x08Operator\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08inputIds\x18\x02 \x03(\t\x12\"\n\x06schema\x18\x03 \x01(\x0b\x32\x12.cz.proto.DataType\x12/\n\x0e\x63olumnMappings\x18\x04 \x03(\x0b\x32\x17.cz.proto.ColumnMapping\x12\x11\n\tsignature\x18\x05 \x01(\x04\x12)\n\ntable_scan\x18\n \x01(\x0b\x32\x13.cz.proto.TableScanH\x00\x12)\n\ntable_sink\x18\x0b \x01(\x0b\x32\x13.cz.proto.TableSinkH\x00\x12\x1e\n\x04\x63\x61lc\x18\x0c \x01(\x0b\x32\x0e.cz.proto.CalcH\x00\x12)\n\nmerge_sort\x18\r \x01(\x0b\x32\x13.cz.proto.MergeSortH\x00\x12/\n\rshuffle_write\x18\x0e \x01(\x0b\x32\x16.cz.proto.ShuffleWriteH\x00\x12-\n\x0cshuffle_read\x18\x0f \x01(\x0b\x32\x15.cz.proto.ShuffleReadH\x00\x12\"\n\x06values\x18\x10 \x01(\x0b\x32\x10.cz.proto.ValuesH\x00\x12+\n\x08hash_agg\x18\x11 \x01(\x0b\x32\x17.cz.proto.HashAggregateH\x00\x12/\n\nsorted_agg\x18\x12 \x01(\x0b\x32\x19.cz.proto.SortedAggregateH\x00\x12-\n\nmerge_join\x18\x13 \x01(\x0b\x32\x17.cz.proto.SortMergeJoinH\x00\x12\'\n\thash_join\x18\x14 \x01(\x0b\x32\x12.cz.proto.HashJoinH\x00\x12)\n\nlocal_sort\x18\x15 \x01(\x0b\x32\x13.cz.proto.LocalSortH\x00\x12\'\n\tunion_all\x18\x16 \x01(\x0b\x32\x12.cz.proto.UnionAllH\x00\x12\"\n\x06\x62uffer\x18\x17 \x01(\x0b\x32\x10.cz.proto.BufferH\x00\x12\"\n\x06window\x18\x18 \x01(\x0b\x32\x10.cz.proto.WindowH\x00\x12\"\n\x06\x65xpand\x18\x19 \x01(\x0b\x32\x10.cz.proto.ExpandH\x00\x12-\n\x0clateral_view\x18\x1a \x01(\x0b\x32\x15.cz.proto.LateralViewH\x00\x12&\n\x08grouping\x18\x1b \x01(\x0b\x32\x12.cz.proto.GroupingH\x00\x12%\n\x04join\x18\x32 \x01(\x0b\x32\x15.cz.proto.LogicalJoinH\x00\x12/\n\taggregate\x18\x33 \x01(\x0b\x32\x1a.cz.proto.LogicalAggregateH\x00\x12-\n\x0clogical_calc\x18\x34 \x01(\x0b\x32\x15.cz.proto.LogicalCalcH\x00\x12-\n\x0clogical_sort\x18\x35 \x01(\x0b\x32\x15.cz.proto.LogicalSortH\x00\x12-\n\x0cset_operator\x18\x36 \x01(\x0b\x32\x15.cz.proto.SetOperatorH\x00\x12-\n\tagg_phase\x18\x37 \x01(\x0b\x32\x18.cz.proto.AggregatePhaseH\x00\x12 \n\x05spool\x18\x38 \x01(\x0b\x32\x0f.cz.proto.SpoolH\x00\x12>\n\x15partial_window_filter\x18\x39 \x01(\x0b\x32\x1d.cz.proto.PartialWindowFilterH\x00\x12\'\n\ttree_join\x18: \x01(\x0b\x32\x12.cz.proto.TreeJoinH\x00\x12\x30\n\x0etree_join_leaf\x18; \x01(\x0b\x32\x16.cz.proto.TreeJoinLeafH\x00\x12\x31\n\x0elocal_exchange\x18< \x01(\x0b\x32\x17.cz.proto.LocalExchangeH\x00\x42\x04\n\x02op\"\xd2\x02\n\rAggregateCall\x12,\n\x08\x66unction\x18\x01 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\x10\n\x08\x64istinct\x18\x02 \x01(\x08\x12!\n\x05stage\x18\x03 \x01(\x0e\x32\x12.cz.proto.AggStage\x12%\n\x06orders\x18\x04 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12#\n\x06\x66ilter\x18\x05 \x01(\x0b\x32\x13.cz.proto.Reference\x12\x15\n\routput_fields\x18\x06 \x03(\x04\x12(\n\x0cinitial_Type\x18\n \x01(\x0b\x32\x12.cz.proto.DataType\x12(\n\x0cpartial_Type\x18\x0b \x01(\x0b\x32\x12.cz.proto.DataType\x12\'\n\x0boutput_Type\x18\x0c \x01(\x0b\x32\x12.cz.proto.DataType\"g\n\x10LogicalAggregate\x12!\n\x04keys\x18\x01 \x03(\x0b\x32\x13.cz.proto.Reference\x12\x30\n\x0f\x61ggregate_calls\x18\x02 \x03(\x0b\x32\x17.cz.proto.AggregateCall\"a\n\rHashAggregate\x12-\n\taggregate\x18\x01 \x01(\x0b\x32\x1a.cz.proto.LogicalAggregate\x12!\n\x05stage\x18\x02 \x01(\x0e\x32\x12.cz.proto.AggStage\"\x8a\x01\n\x0fSortedAggregate\x12-\n\taggregate\x18\x01 \x01(\x0b\x32\x1a.cz.proto.LogicalAggregate\x12!\n\x05stage\x18\x02 \x01(\x0e\x32\x12.cz.proto.AggStage\x12%\n\x06orders\x18\x03 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\"b\n\x0e\x41ggregatePhase\x12-\n\taggregate\x18\x01 \x01(\x0b\x32\x1a.cz.proto.LogicalAggregate\x12!\n\x05stage\x18\x02 \x01(\x0e\x32\x12.cz.proto.AggStage\"\xaa\x01\n\x11\x44ynamicFilterInfo\x12)\n\x04type\x18\x01 \x01(\x0e\x32\x1b.cz.proto.DynamicFilterType\x12\x10\n\x08\x63onsumer\x18\x02 \x01(\x08\x12\x13\n\x0bselectivity\x18\x03 \x01(\x01\x12\r\n\x05probe\x18\x04 \x01(\r\x12\x18\n\x10partition_filter\x18\x05 \x01(\x08\x12\x1a\n\x12table_scan_parents\x18\x06 \x01(\r\")\n\x0cJoinHintInfo\x12\x19\n\x11hintBroadcastSide\x18\x01 \x01(\x05\"6\n\x08TreeJoin\x12*\n\x0eroot_operators\x18\x01 \x03(\x0b\x32\x12.cz.proto.Operator\";\n\x0cTreeJoinLeaf\x12\x13\n\x0binput_index\x18\x01 \x01(\x05\x12\x16\n\x0ehint_broadcast\x18\x02 \x01(\x08\"\xd7\x01\n\x0bLogicalJoin\x12 \n\x04type\x18\x01 \x01(\x0e\x32\x12.cz.proto.JoinType\x12-\n\tcondition\x18\x02 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\x18\n\x10input_references\x18\x03 \x03(\x04\x12\x33\n\x0e\x64ynamic_filter\x18\x04 \x01(\x0b\x32\x1b.cz.proto.DynamicFilterInfo\x12(\n\x08hintInfo\x18\x05 \x01(\x0b\x32\x16.cz.proto.JoinHintInfo\"\x8a\x01\n\rSortMergeJoin\x12#\n\x04join\x18\x01 \x01(\x0b\x32\x15.cz.proto.LogicalJoin\x12)\n\nlhs_orders\x18\x02 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12)\n\nrhs_orders\x18\x03 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\"]\n\x08HashJoin\x12#\n\x04join\x18\x01 \x01(\x0b\x32\x15.cz.proto.LogicalJoin\x12\x19\n\x11probe_operator_id\x18\x02 \x01(\t\x12\x11\n\tbroadcast\x18\x03 \x01(\x08\"/\n\x06Timing\x12\x11\n\tcpu_nanos\x18\x01 \x01(\x04\x12\x12\n\nwall_nanos\x18\x02 \x01(\x04\"\xe3\n\n\rOperatorStats\x12\x13\n\x0boperator_id\x18\x01 \x01(\t\x12\x11\n\trow_count\x18\x02 \x01(\x04\x12 \n\x06timing\x18\x03 \x01(\x0b\x32\x10.cz.proto.Timing\x12\x34\n\x10table_scan_stats\x18\x04 \x01(\x0b\x32\x18.cz.proto.TableScanStatsH\x00\x12\x34\n\x10table_sink_stats\x18\x05 \x01(\x0b\x32\x18.cz.proto.TableSinkStatsH\x00\x12)\n\ncalc_stats\x18\x06 \x01(\x0b\x32\x13.cz.proto.CalcStatsH\x00\x12\x32\n\x0fhash_join_stats\x18\x07 \x01(\x0b\x32\x17.cz.proto.HashJoinStatsH\x00\x12\x34\n\x10merge_join_stats\x18\x08 \x01(\x0b\x32\x18.cz.proto.MergeJoinStatsH\x00\x12<\n\x14hash_aggregate_stats\x18\t \x01(\x0b\x32\x1c.cz.proto.HashAggregateStatsH\x00\x12>\n\x15merge_aggregate_stats\x18\n \x01(\x0b\x32\x1d.cz.proto.MergeAggregateStatsH\x00\x12\x34\n\x10local_sort_stats\x18\x0b \x01(\x0b\x32\x18.cz.proto.LocalSortStatsH\x00\x12\x34\n\x10merge_sort_stats\x18\x0c \x01(\x0b\x32\x18.cz.proto.MergeSortStatsH\x00\x12-\n\x0cvalues_stats\x18\r \x01(\x0b\x32\x15.cz.proto.ValuesStatsH\x00\x12:\n\x13\x65xchange_sink_stats\x18\x0e \x01(\x0b\x32\x1b.cz.proto.ExchangeSinkStatsH\x00\x12>\n\x15\x65xchange_source_stats\x18\x0f \x01(\x0b\x32\x1d.cz.proto.ExchangeSourceStatsH\x00\x12\x32\n\x0funion_all_stats\x18\x10 \x01(\x0b\x32\x17.cz.proto.UnionAllStatsH\x00\x12-\n\x0c\x62uffer_stats\x18\x11 \x01(\x0b\x32\x15.cz.proto.BufferStatsH\x00\x12-\n\x0cwindow_stats\x18\x12 \x01(\x0b\x32\x15.cz.proto.WindowStatsH\x00\x12-\n\x0c\x65xpand_stats\x18\x13 \x01(\x0b\x32\x15.cz.proto.ExpandStatsH\x00\x12\x38\n\x12lateral_view_stats\x18\x14 \x01(\x0b\x32\x1a.cz.proto.LateralViewStatsH\x00\x12<\n\x14partial_window_stats\x18\x15 \x01(\x0b\x32\x1c.cz.proto.PartialWindowStatsH\x00\x12%\n\x0binit_timing\x18\x64 \x01(\x0b\x32\x10.cz.proto.Timing\x12\x13\n\x0b\x62\x61tch_count\x18\x65 \x01(\x04\x12\x13\n\x0bpeak_memory\x18\x66 \x01(\x04\x12\x18\n\x10start_time_nanos\x18g \x01(\x04\x12\x16\n\x0e\x65nd_time_nanos\x18h \x01(\x04\x12\x17\n\x0f\x62\x61tch_signature\x18i \x01(\x04\x12\x14\n\x0c\x65xec_node_id\x18j \x01(\t\x12\x13\n\x0bpipeline_id\x18k \x01(\x04\x12\x17\n\x0f\x64river_sequence\x18l \x01(\x04\x12\x1a\n\x12\x62lock_timing_nanos\x18m \x01(\x04\x12\x1b\n\x12\x65xtra_stats_binary\x18\xfe\x01 \x01(\x0c\x12\x14\n\x0b\x65xtra_stats\x18\xff\x01 \x01(\tB\n\n\x08op_stats\"U\n\x0bOrderByDesc\x12&\n\treference\x18\x01 \x01(\x0b\x32\x13.cz.proto.Reference\x12\x1e\n\x05order\x18\x02 \x01(\x0e\x32\x0f.cz.proto.Order\"L\n\tMergeSort\x12%\n\x06orders\x18\x01 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12\x18\n\x10input_references\x18\x02 \x03(\x04\"$\n\x08UnionAll\x12\x18\n\x10input_references\x18\x01 \x03(\x04\"\x18\n\x06\x42uffer\x12\x0e\n\x06shared\x18\x01 \x01(\x08\"v\n\x13PartialWindowFilter\x12,\n\x08\x66unction\x18\x01 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\"\n\x04spec\x18\x02 \x01(\x0b\x32\x14.cz.proto.WindowSpec\x12\r\n\x05limit\x18\x03 \x01(\x04\"I\n\x06Window\x12%\n\x06groups\x18\x01 \x03(\x0b\x32\x15.cz.proto.WindowGroup\x12\x18\n\x10input_references\x18\x02 \x03(\x04\"Z\n\x0bWindowGroup\x12\'\n\tfunctions\x18\x01 \x03(\x0b\x32\x14.cz.proto.WindowCall\x12\"\n\x04spec\x18\x02 \x01(\x0b\x32\x14.cz.proto.WindowSpec\"v\n\nWindowCall\x12,\n\x08\x66unction\x18\x01 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\x10\n\x08\x64istinct\x18\x02 \x01(\x08\x12(\n\x0cpartial_type\x18\x03 \x01(\x0b\x32\x12.cz.proto.DataType\"\x9e\x02\n\nWindowSpec\x12!\n\x04keys\x18\x01 \x03(\x0b\x32\x13.cz.proto.Reference\x12%\n\x06orders\x18\x02 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12\x38\n\rboundary_type\x18\x03 \x01(\x0e\x32!.cz.proto.WindowSpec.BoundaryType\x12-\n\x0blower_bound\x18\x04 \x01(\x0b\x32\x18.cz.proto.WindowBoundary\x12-\n\x0bupper_bound\x18\x05 \x01(\x0b\x32\x18.cz.proto.WindowBoundary\".\n\x0c\x42oundaryType\x12\x08\n\x04ROWS\x10\x00\x12\t\n\x05RANGE\x10\x01\x12\t\n\x05GROUP\x10\x02\"Z\n\x0eWindowBoundary\x12\x16\n\tpreceding\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\"\n\x06offset\x18\x02 \x01(\x0b\x32\x12.cz.proto.ConstantB\x0c\n\n_preceding\"W\n\x0bLateralView\x12.\n\tfunctions\x18\x01 \x03(\x0b\x32\x1b.cz.proto.TableFunctionCall\x12\x18\n\x10input_references\x18\x02 \x03(\x04\"e\n\x11TableFunctionCall\x12,\n\x08\x66unction\x18\x01 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\r\n\x05outer\x18\x02 \x01(\x08\x12\x13\n\x0bused_fields\x18\x03 \x03(\x04\"\x19\n\x05Spool\x12\x10\n\x08spool_id\x18\x01 \x01(\x04\"\xfb\x01\n\x04\x43\x61lc\x12/\n\x0b\x65xpressions\x18\x01 \x03(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\x13\n\tno_filter\x18\x02 \x01(\x08H\x00\x12\x10\n\x06\x66ilter\x18\x03 \x01(\x04H\x00\x12\x10\n\x08projects\x18\x04 \x03(\x04\x12 \n\x04lazy\x18\x05 \x03(\x0e\x32\x12.cz.proto.LazyEval\x12.\n!partial_window_filter_selectivity\x18\x06 \x01(\x01H\x01\x88\x01\x01\x42\x11\n\x0foptional_filterB$\n\"_partial_window_filter_selectivity\"\xc0\x01\n\x0bLogicalCalc\x12-\n\tcondition\x18\x01 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12,\n\x08projects\x18\x02 \x03(\x0b\x32\x1a.cz.proto.ScalarExpression\x12.\n!partial_window_filter_selectivity\x18\x03 \x01(\x01H\x00\x88\x01\x01\x42$\n\"_partial_window_filter_selectivity\"9\n\x06\x45xpand\x12/\n\x0b\x65xpressions\x18\x01 \x03(\x0b\x32\x1a.cz.proto.ScalarExpression\"3\n\x0eGroupingKeySet\x12!\n\x04keys\x18\x01 \x03(\x0b\x32\x13.cz.proto.Reference\"\xea\x01\n\x08Grouping\x12!\n\x04keys\x18\x01 \x03(\x0b\x32\x13.cz.proto.Reference\x12)\n\x07keySets\x18\x02 \x03(\x0b\x32\x18.cz.proto.GroupingKeySet\x12\x30\n\x0f\x61ggregate_calls\x18\x03 \x03(\x0b\x32\x17.cz.proto.AggregateCall\x12\x1e\n\x16grouping_id_start_from\x18\x04 \x01(\x04\x12#\n\x16grouping_id_col_offset\x18\x05 \x01(\x04H\x00\x88\x01\x01\x42\x19\n\x17_grouping_id_col_offset\"\x0b\n\tCalcStats\"\x8e\x01\n\x0eHashTableStats\x12\x13\n\x0bnum_buckets\x18\x01 \x01(\x04\x12\x10\n\x08num_keys\x18\x02 \x01(\x04\x12\x12\n\nnum_resize\x18\x03 \x01(\x04\x12\x14\n\x0cnum_accesses\x18\x04 \x01(\x04\x12\x16\n\x0enum_collisions\x18\x05 \x01(\x04\x12\x13\n\x0bused_memory\x18\x06 \x01(\x04\"\xed\x04\n\rHashJoinStats\x12&\n\x0c\x62uild_timing\x18\x01 \x01(\x0b\x32\x10.cz.proto.Timing\x12-\n\x13\x66inish_build_timing\x18\x02 \x01(\x0b\x32\x10.cz.proto.Timing\x12&\n\x0cprobe_timing\x18\x03 \x01(\x0b\x32\x10.cz.proto.Timing\x12+\n\x11post_probe_timing\x18\x04 \x01(\x0b\x32\x10.cz.proto.Timing\x12*\n\x08ht_stats\x18\x05 \x01(\x0b\x32\x18.cz.proto.HashTableStats\x12\x16\n\x0enum_build_rows\x18\x06 \x01(\x04\x12\x1f\n\x17num_distinct_build_rows\x18\x07 \x01(\x04\x12\x1c\n\x14max_equal_build_rows\x18\x08 \x01(\x04\x12/\n\x11\x62uild_spill_stats\x18\t \x01(\x0b\x32\x14.cz.proto.SpillStats\x12/\n\x11probe_spill_stats\x18\n \x01(\x0b\x32\x14.cz.proto.SpillStats\x12.\n\x14probe_find_ht_timing\x18\x0b \x01(\x0b\x32\x10.cz.proto.Timing\x12-\n\x13probe_output_timing\x18\x0c \x01(\x0b\x32\x10.cz.proto.Timing\x12\x34\n\x1aprobe_eval_conjunct_timing\x18\r \x01(\x0b\x32\x10.cz.proto.Timing\x12\x36\n\x1cprobe_output_conjunct_timing\x18\x0e \x01(\x0b\x32\x10.cz.proto.Timing\"\x10\n\x0eMergeJoinStats\"\xe7\x02\n\x12HashAggregateStats\x12.\n\x14\x61ssign_states_timing\x18\x01 \x01(\x0b\x32\x10.cz.proto.Timing\x12.\n\x14update_states_timing\x18\x02 \x01(\x0b\x32\x10.cz.proto.Timing\x12\'\n\routput_timing\x18\x03 \x01(\x0b\x32\x10.cz.proto.Timing\x12*\n\x08ht_stats\x18\x04 \x01(\x0b\x32\x18.cz.proto.HashTableStats\x12\x1a\n\x12states_used_memory\x18\x05 \x01(\x04\x12\x19\n\x11pass_through_rows\x18\x06 \x01(\x04\x12/\n\x11input_spill_stats\x18\x07 \x01(\x0b\x32\x14.cz.proto.SpillStats\x12\x34\n\x16\x61ggregated_spill_stats\x18\x08 \x01(\x0b\x32\x14.cz.proto.SpillStats\"\x9e\x01\n\x13MergeAggregateStats\x12.\n\x14\x61ssign_states_timing\x18\x01 \x01(\x0b\x32\x10.cz.proto.Timing\x12.\n\x14update_states_timing\x18\x02 \x01(\x0b\x32\x10.cz.proto.Timing\x12\'\n\routput_timing\x18\x03 \x01(\x0b\x32\x10.cz.proto.Timing\"\xff\x02\n\x0eLocalSortStats\x12)\n\x0bspill_stats\x18\x01 \x01(\x0b\x32\x14.cz.proto.SpillStats\x12-\n\x13generate_run_timing\x18\x02 \x01(\x0b\x32\x10.cz.proto.Timing\x12*\n\x10merge_run_timing\x18\x03 \x01(\x0b\x32\x10.cz.proto.Timing\x12+\n\x11init_merge_timing\x18\x04 \x01(\x0b\x32\x10.cz.proto.Timing\x12\x31\n\x17\x61\x63\x63umulate_block_timing\x18\x05 \x01(\x0b\x32\x10.cz.proto.Timing\x12)\n\x0fsort_key_timing\x18\x06 \x01(\x0b\x32\x10.cz.proto.Timing\x12\x30\n\x16permute_payload_timing\x18\x07 \x01(\x0b\x32\x10.cz.proto.Timing\x12*\n\x10spill_run_timing\x18\x08 \x01(\x0b\x32\x10.cz.proto.Timing\"\x10\n\x0eMergeSortStats\"\x0f\n\rUnionAllStats\"\r\n\x0bValuesStats\"8\n\x0b\x42ufferStats\x12)\n\x0bspill_stats\x18\x01 \x01(\x0b\x32\x14.cz.proto.SpillStats\"\r\n\x0bWindowStats\"\x14\n\x12PartialWindowStats\"\r\n\x0b\x45xpandStats\"\x12\n\x10LateralViewStats\"\x9d\x03\n\x11\x45xchangeSinkStats\x12\x17\n\x0fsent_byte_count\x18\x01 \x01(\x04\x12!\n\x19\x63ompress_input_byte_count\x18\x02 \x01(\x04\x12\x30\n\x16serialize_write_timing\x18\x03 \x01(\x0b\x32\x10.cz.proto.Timing\x12\x30\n\x16serialize_flush_timing\x18\x04 \x01(\x0b\x32\x10.cz.proto.Timing\x12)\n\x0f\x63ompress_timing\x18\x05 \x01(\x0b\x32\x10.cz.proto.Timing\x12/\n\x15\x61\x63quire_buffer_timing\x18\x06 \x01(\x0b\x32\x10.cz.proto.Timing\x12.\n\x14submit_buffer_timing\x18\x07 \x01(\x0b\x32\x10.cz.proto.Timing\x12&\n\x0c\x63lose_timing\x18\x08 \x01(\x0b\x32\x10.cz.proto.Timing\x12\x34\n\x1asubmit_buffer_async_timing\x18\t \x01(\x0b\x32\x10.cz.proto.Timing\"\xba\x02\n\x13\x45xchangeSourceStats\x12\x1b\n\x13received_byte_count\x18\x01 \x01(\x04\x12$\n\x1c\x64\x65\x63ompress_output_byte_count\x18\x02 \x01(\x04\x12,\n\x12\x64\x65serialize_timing\x18\x03 \x01(\x0b\x32\x10.cz.proto.Timing\x12+\n\x11\x64\x65\x63ompress_timing\x18\x04 \x01(\x0b\x32\x10.cz.proto.Timing\x12,\n\x12read_buffer_timing\x18\x05 \x01(\x0b\x32\x10.cz.proto.Timing\x12%\n\x0bsort_timing\x18\x06 \x01(\x0b\x32\x10.cz.proto.Timing\x12\x30\n\x12sorter_spill_stats\x18\x07 \x01(\x0b\x32\x14.cz.proto.SpillStats\"S\n\x05Table\x12\x0c\n\x04path\x18\x01 \x03(\t\x12\'\n\ntable_meta\x18\x02 \x01(\x0b\x32\x13.cz.proto.TableMeta\x12\x13\n\x0binstance_id\x18\x03 \x01(\x03\"k\n\x06Values\x12\x11\n\trow_count\x18\x01 \x01(\r\x12\x11\n\tcol_count\x18\x02 \x01(\r\x12(\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\x11\n\tbroadcast\x18\x04 \x01(\x08\"9\n\x0bSortKeyDesc\x12\n\n\x02id\x18\x01 \x01(\r\x12\x1e\n\x05order\x18\x02 \x01(\x0e\x32\x0f.cz.proto.Order\"\xf6\x03\n\tTableScan\x12\x1e\n\x05table\x18\x01 \x01(\x0b\x32\x0f.cz.proto.Table\x12\x1b\n\x13\x64\x61ta_source_info_id\x18\x02 \x01(\x05\x12\x0c\n\x04\x63ols\x18\x03 \x03(\x04\x12*\n\x06\x66ilter\x18\x04 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\x31\n\rensuredFilter\x18\x05 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12-\n\x05props\x18\x06 \x03(\x0b\x32\x1e.cz.proto.TableScan.PropsEntry\x12\r\n\x05\x61lign\x18\x07 \x01(\x08\x12\x10\n\x08\x61lignDop\x18\x08 \x01(\x04\x12%\n\x06orders\x18\t \x03(\x0b\x32\x15.cz.proto.SortKeyDesc\x12\x12\n\nrange_keys\x18\n \x03(\r\x12\x1d\n\x15range_distribution_id\x18\x0b \x01(\r\x12\x36\n\x12tvr_table_property\x18\x0c \x01(\x0b\x32\x1a.cz.proto.TvrTableProperty\x12/\n\x0b\x66ilter4Meta\x18\r \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x1a,\n\nPropsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"?\n\x0eTableScanStats\x12-\n\x0binput_stats\x18\x01 \x01(\x0b\x32\x18.cz.proto.DataInputStats\"\x99\x01\n\x10TvrTableProperty\x12\x0c\n\x04\x66rom\x18\x01 \x01(\x03\x12\n\n\x02to\x18\x02 \x01(\x03\x12\x13\n\x0b\x63onsolidate\x18\x03 \x01(\x08\x12\x17\n\x0f\x66romMetaVersion\x18\x04 \x01(\x03\x12\x15\n\rtoMetaVersion\x18\x05 \x01(\x03\x12\x10\n\x08rowCount\x18\x06 \x01(\x03\x12\x14\n\x0c\x62\x61seRowCount\x18\x07 \x01(\x03\"\xc3\x01\n\x0bShuffleType\x12(\n\x04type\x18\x01 \x01(\x0e\x32\x1a.cz.proto.ShuffleType.Type\"\x89\x01\n\x04Type\x12\x08\n\x04HASH\x10\x00\x12\t\n\x05RANGE\x10\x01\x12\r\n\tBROADCAST\x10\x02\x12\n\n\x06SINGLE\x10\x03\x12\n\n\x06RANDOM\x10\x04\x12\r\n\tPAIR_WIZE\x10\x05\x12\x11\n\rADAPTIVE_HASH\x10\x06\x12\x12\n\x0e\x41\x44\x41PTIVE_RANGE\x10\x07\x12\x0f\n\x0bROUND_ROBIN\x10\x08\"\xd6\x02\n\tTableSink\x12\x1e\n\x05table\x18\x01 \x01(\x0b\x32\x0f.cz.proto.Table\x12\x11\n\toverwrite\x18\x02 \x01(\x08\x12\x1b\n\x13\x64\x61ta_source_info_id\x18\x03 \x01(\x05\x12!\n\x04keys\x18\x04 \x03(\x0b\x32\x13.cz.proto.Reference\x12\r\n\x05\x66lags\x18\x05 \x01(\r\x12+\n\x0epart_sort_keys\x18\x06 \x03(\x0b\x32\x13.cz.proto.Reference\x12\x14\n\x0cinput_fields\x18\x07 \x03(\x04\x12,\n\x0f\x66ile_slice_keys\x18\x08 \x03(\x0b\x32\x13.cz.proto.Reference\x12\x18\n\x10static_partition\x18\t \x01(\x08\x12/\n\x0bpart_values\x18\n \x01(\x0b\x32\x1a.cz.proto.VirtualValueInfo\x12\x0b\n\x03nop\x18\x0b \x01(\x08\"\xfb\x01\n\x0cShuffleWrite\x12*\n\x0bshuffleType\x18\x01 \x01(\x0b\x32\x15.cz.proto.ShuffleType\x12!\n\x04keys\x18\x02 \x03(\x0b\x32\x13.cz.proto.Reference\x12%\n\x06orders\x18\x03 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12\r\n\x05limit\x18\x04 \x01(\r\x12\x18\n\x10\x66unction_version\x18\x05 \x01(\r\x12-\n\x0b\x62ucket_type\x18\x06 \x01(\x0e\x32\x18.cz.proto.HashBucketType\x12\x1d\n\x15range_distribution_id\x18\x07 \x01(\r\"\xbe\x01\n\x0bShuffleRead\x12%\n\x06orders\x18\x01 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12\r\n\x05limit\x18\x02 \x01(\r\x12\x0e\n\x06offset\x18\x03 \x01(\r\x12*\n\x0bshuffleType\x18\x04 \x01(\x0b\x32\x15.cz.proto.ShuffleType\x12\x16\n\x0emulti_accessed\x18\x05 \x01(\x08\x12\x11\n\tpre_start\x18\x06 \x01(\x08\x12\x12\n\nmerge_sort\x18\x07 \x01(\x08\"\xbe\x01\n\rLocalExchange\x12*\n\x0bshuffleType\x18\x01 \x01(\x0b\x32\x15.cz.proto.ShuffleType\x12!\n\x04keys\x18\x02 \x03(\x0b\x32\x13.cz.proto.Reference\x12%\n\x06orders\x18\x03 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12\r\n\x05limit\x18\x04 \x01(\r\x12\x0e\n\x06offset\x18\x05 \x01(\r\x12\x18\n\x10\x66unction_version\x18\x06 \x01(\r\"Q\n\tLocalSort\x12%\n\x06orders\x18\x01 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12\r\n\x05limit\x18\x02 \x01(\x04\x12\x0e\n\x06offset\x18\x03 \x01(\x04\"\xbe\x01\n\x0bLogicalSort\x12!\n\x04keys\x18\x01 \x03(\x0b\x32\x13.cz.proto.Reference\x12%\n\x06orders\x18\x02 \x03(\x0b\x32\x15.cz.proto.OrderByDesc\x12)\n\x05limit\x18\x03 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12*\n\x06offset\x18\x04 \x01(\x0b\x32\x1a.cz.proto.ScalarExpression\x12\x0e\n\x06global\x18\x05 \x01(\x08\"W\n\x0bSetOperator\x12!\n\x04type\x18\x01 \x01(\x0e\x32\x13.cz.proto.SetOpType\x12\x0b\n\x03\x61ll\x18\x02 \x01(\x08\x12\x18\n\x10input_references\x18\x03 \x03(\x04\"\xdc\x01\n\x0e\x44\x61taInputStats\x12\x1c\n\x14raw_input_byte_count\x18\x01 \x01(\x04\x12\x11\n\trow_count\x18\x02 \x01(\x04\x12:\n\x10\x66ile_input_stats\x18\x03 \x01(\x0b\x32\x1e.cz.proto.FileRangesInputStatsH\x00\x12\x35\n\x11\x66ile_format_stats\x18\x04 \x01(\x0b\x32\x18.cz.proto.FileInputStatsH\x00\x12\x17\n\x0ftime_elapsed_us\x18\x05 \x01(\x04\x42\r\n\x0binput_stats\"J\n\x14\x46ileRangesInputStats\x12\x32\n\x10\x66ile_input_stats\x18\x01 \x03(\x0b\x32\x18.cz.proto.DataInputStats\"\x9f\x05\n\x0e\x46ileInputStats\x12-\n\x0b\x66ormat_type\x18\x01 \x01(\x0e\x32\x18.cz.proto.FileFormatType\x12\"\n\x05range\x18\x02 \x01(\x0b\x32\x13.cz.proto.FileRange\x12\x34\n\x10text_input_stats\x18\x03 \x01(\x0b\x32\x18.cz.proto.TextInputStatsH\x00\x12:\n\x13parquet_input_stats\x18\x04 \x01(\x0b\x32\x1b.cz.proto.ParquetInputStatsH\x00\x12=\n\x12memory_input_stats\x18\x05 \x01(\x0b\x32\x1f.cz.proto.MemoryTableInputStatsH\x00\x12\x32\n\x0forc_input_stats\x18\x08 \x01(\x0b\x32\x17.cz.proto.OrcInputStatsH\x00\x12\x36\n\x11\x64ummy_input_stats\x18\t \x01(\x0b\x32\x19.cz.proto.DummyInputStatsH\x00\x12\x32\n\x0f\x63sv_input_stats\x18\n \x01(\x0b\x32\x17.cz.proto.CSVInputStatsH\x00\x12\x34\n\x10\x61vro_input_stats\x18\x0b \x01(\x0b\x32\x18.cz.proto.AvroInputStatsH\x00\x12\x36\n\x11\x61rrow_input_stats\x18\x0c \x01(\x0b\x32\x19.cz.proto.ArrowInputStatsH\x00\x12,\n\x08io_stats\x18\x06 \x01(\x0b\x32\x1a.cz.proto.FileIOInputStats\x12\x32\n\x10\x64\x65lta_file_stats\x18\x07 \x03(\x0b\x32\x18.cz.proto.DataInputStatsB\x19\n\x17\x66ile_format_input_stats\"\xb9\x01\n\x10\x46ileIOInputStats\x12\x12\n\nread_count\x18\x01 \x01(\x04\x12\x12\n\nread_bytes\x18\x02 \x01(\x04\x12\x17\n\x0ftime_elapsed_us\x18\x03 \x01(\x04\x12/\n\x0eprefetch_stats\x18\x04 \x01(\x0b\x32\x17.cz.proto.PrefetchStats\x12\x33\n\x0binput_stats\x18\x05 \x01(\x0b\x32\x1e.cz.proto.FileInputStreamStats\",\n\x11\x46ileIOOutputStats\x12\x17\n\x0ftime_elapsed_us\x18\x01 \x01(\x04\"\xc2\x01\n\rPrefetchStats\x12\x13\n\x0b\x64river_type\x18\x08 \x01(\t\x12\x12\n\nread_count\x18\x01 \x01(\x04\x12\x12\n\nread_bytes\x18\x02 \x01(\x04\x12\x16\n\x0eread_hit_cache\x18\x03 \x01(\x04\x12\x1c\n\x14read_time_elapsed_us\x18\x04 \x01(\x04\x12\x10\n\x08io_count\x18\x05 \x01(\x04\x12\x10\n\x08io_bytes\x18\x06 \x01(\x04\x12\x1a\n\x12io_time_elapsed_us\x18\x07 \x01(\x04\"\xb3\x01\n\x14\x46ileInputStreamStats\x12\x32\n\x10\x66ile_system_type\x18\x01 \x01(\x0e\x32\x18.cz.proto.FileSystemType\x12L\n\x1d\x63\x61\x63he_file_input_stream_stats\x18\x04 \x01(\x0b\x32#.cz.proto.CacheFileInputStreamStatsH\x00\x42\x19\n\x17\x66ile_input_stream_stats\"\x83\x02\n\x19\x43\x61\x63heFileInputStreamStats\x12\x11\n\tcache_hit\x18\x01 \x01(\x08\x12!\n\x19short_circuit_stream_type\x18\x02 \x01(\t\x12\x14\n\x0csegment_mode\x18\x03 \x01(\x08\x12 \n\x18non_read_time_elapsed_us\x18\x04 \x01(\x04\x12\x16\n\x0erpc_read_bytes\x18\x05 \x01(\x04\x12 \n\x18rpc_read_time_elapsed_us\x18\x06 \x01(\x04\x12\x19\n\x11\x64irect_read_bytes\x18\x07 \x01(\x04\x12#\n\x1b\x64irect_read_time_elapsed_us\x18\x08 \x01(\x04\"V\n\x0eTextInputStats\x12\"\n\x1amissing_field_warned_count\x18\x01 \x01(\x04\x12 \n\x18\x65xtra_field_warned_count\x18\x02 \x01(\x04\",\n\x15MemoryTableInputStats\x12\x13\n\x0b\x62\x61tch_count\x18\x01 \x01(\x04\"\x8b\x02\n\x11ParquetInputStats\x12\x13\n\x0b\x62\x61tch_count\x18\x01 \x01(\x04\x12 \n\x18\x64\x65\x63ompression_latency_ns\x18\x02 \x01(\x04\x12\"\n\x1alevels_decoding_latency_ns\x18\x03 \x01(\x04\x12\x1f\n\x17\x64\x61ta_loading_latency_ns\x18\x04 \x01(\x04\x12 \n\x18ppd_inclusive_latency_ns\x18\x05 \x01(\x04\x12#\n\x1breader_inclusive_latency_ns\x18\x06 \x01(\x04\x12\x1b\n\x13requested_row_count\x18\x07 \x01(\x04\x12\x16\n\x0eread_row_count\x18\x08 \x01(\x04\"$\n\rOrcInputStats\x12\x13\n\x0b\x62\x61tch_count\x18\x01 \x01(\x04\"\x0f\n\rCSVInputStats\"&\n\x0f\x44ummyInputStats\x12\x13\n\x0b\x62\x61tch_count\x18\x01 \x01(\x04\"\x10\n\x0e\x41vroInputStats\"\x11\n\x0f\x41rrowInputStats\"\xd3\x01\n\x0f\x44\x61taOutputStats\x12\x1d\n\x15raw_output_byte_count\x18\x01 \x01(\x04\x12\x11\n\trow_count\x18\x02 \x01(\x04\x12\x36\n\x11\x66ile_output_stats\x18\x03 \x01(\x0b\x32\x19.cz.proto.FileOutputStatsH\x00\x12G\n\x1amultiple_file_output_stats\x18\x04 \x01(\x0b\x32!.cz.proto.MultipleFileOutputStatsH\x00\x42\r\n\x0boutputStats\"\xdd\x03\n\x0f\x46ileOutputStats\x12.\n\x0e\x66ile_meta_data\x18\x01 \x01(\x0b\x32\x16.cz.proto.FileMetaData\x12\x36\n\x11text_output_stats\x18\x02 \x01(\x0b\x32\x19.cz.proto.TextOutputStatsH\x00\x12<\n\x14parquet_output_stats\x18\x03 \x01(\x0b\x32\x1c.cz.proto.ParquetOutputStatsH\x00\x12\x34\n\x10orc_output_stats\x18\x06 \x01(\x0b\x32\x18.cz.proto.OrcOutputStatsH\x00\x12\x36\n\x11\x61vro_output_stats\x18\x07 \x01(\x0b\x32\x19.cz.proto.AvroOutputStatsH\x00\x12\x38\n\x12\x61rrow_output_stats\x18\x08 \x01(\x0b\x32\x1a.cz.proto.ArrowOutputStatsH\x00\x12-\n\x08io_stats\x18\x04 \x01(\x0b\x32\x1b.cz.proto.FileIOOutputStats\x12\x31\n\x11\x64\x65lete_file_metas\x18\x05 \x03(\x0b\x32\x16.cz.proto.FileMetaDataB\x1a\n\x18\x66ile_format_output_stats\"O\n\x17MultipleFileOutputStats\x12\x34\n\x11\x66ile_output_stats\x18\x01 \x03(\x0b\x32\x19.cz.proto.DataOutputStats\"\x11\n\x0fTextOutputStats\"c\n\x12ParquetOutputStats\x12\x1a\n\x12\x61rrow_casting_time\x18\x01 \x01(\x04\x12\x1a\n\x12page_compress_time\x18\x02 \x01(\x04\x12\x15\n\rencoding_time\x18\x03 \x01(\x04\"\x10\n\x0eOrcOutputStats\"\x11\n\x0f\x41vroOutputStats\"\x12\n\x10\x41rrowOutputStats\"A\n\x0eTableSinkStats\x12/\n\x0coutput_stats\x18\x01 \x01(\x0b\x32\x19.cz.proto.DataOutputStats\"_\n\nSpillStats\x12\x17\n\x0f\x63ompressed_size\x18\x01 \x01(\x04\x12\x10\n\x08raw_size\x18\x02 \x01(\x04\x12\x13\n\x0bspill_count\x18\x03 \x01(\x04\x12\x11\n\trow_count\x18\x04 \x01(\x04*N\n\x08\x41ggStage\x12\r\n\tDUPLICATE\x10\x00\x12\x0c\n\x08PARTIAL1\x10\x01\x12\x0c\n\x08PARTIAL2\x10\x02\x12\t\n\x05\x46INAL\x10\x03\x12\x0c\n\x08\x43OMPLETE\x10\x04*R\n\x08JoinType\x12\t\n\x05INNER\x10\x00\x12\x08\n\x04LEFT\x10\x01\x12\t\n\x05RIGHT\x10\x02\x12\x08\n\x04\x46ULL\x10\x03\x12\r\n\tLEFT_SEMI\x10\x04\x12\r\n\tLEFT_ANTI\x10\x05*R\n\x11\x44ynamicFilterType\x12\x0b\n\x07\x44\x46_NONE\x10\x00\x12\r\n\tDF_GLOBAL\x10\x01\x12\x10\n\x0c\x44\x46_BROADCAST\x10\x02\x12\x0f\n\x0b\x44\x46_SHUFFLED\x10\x03*@\n\x08LazyEval\x12\x0c\n\x08NOT_LAZY\x10\x00\x12\x15\n\x11LAZY_IN_CONDITION\x10\x01\x12\x0f\n\x0b\x41LWAYS_LAZY\x10\x02*1\n\tSetOpType\x12\t\n\x05UNION\x10\x00\x12\r\n\tINTERSECT\x10\x01\x12\n\n\x06\x45XCEPT\x10\x02\x42\x11\x42\rOperatorProtoP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'operator_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\rOperatorProtoP\001'
  _TABLESCAN_PROPSENTRY._options = None
  _TABLESCAN_PROPSENTRY._serialized_options = b'8\001'
  _AGGSTAGE._serialized_start=15373
  _AGGSTAGE._serialized_end=15451
  _JOINTYPE._serialized_start=15453
  _JOINTYPE._serialized_end=15535
  _DYNAMICFILTERTYPE._serialized_start=15537
  _DYNAMICFILTERTYPE._serialized_end=15619
  _LAZYEVAL._serialized_start=15621
  _LAZYEVAL._serialized_end=15685
  _SETOPTYPE._serialized_start=15687
  _SETOPTYPE._serialized_end=15736
  _COLUMNMAPPING._serialized_start=211
  _COLUMNMAPPING._serialized_end=261
  _OPERATOR._serialized_start=264
  _OPERATOR._serialized_end=1684
  _AGGREGATECALL._serialized_start=1687
  _AGGREGATECALL._serialized_end=2025
  _LOGICALAGGREGATE._serialized_start=2027
  _LOGICALAGGREGATE._serialized_end=2130
  _HASHAGGREGATE._serialized_start=2132
  _HASHAGGREGATE._serialized_end=2229
  _SORTEDAGGREGATE._serialized_start=2232
  _SORTEDAGGREGATE._serialized_end=2370
  _AGGREGATEPHASE._serialized_start=2372
  _AGGREGATEPHASE._serialized_end=2470
  _DYNAMICFILTERINFO._serialized_start=2473
  _DYNAMICFILTERINFO._serialized_end=2643
  _JOINHINTINFO._serialized_start=2645
  _JOINHINTINFO._serialized_end=2686
  _TREEJOIN._serialized_start=2688
  _TREEJOIN._serialized_end=2742
  _TREEJOINLEAF._serialized_start=2744
  _TREEJOINLEAF._serialized_end=2803
  _LOGICALJOIN._serialized_start=2806
  _LOGICALJOIN._serialized_end=3021
  _SORTMERGEJOIN._serialized_start=3024
  _SORTMERGEJOIN._serialized_end=3162
  _HASHJOIN._serialized_start=3164
  _HASHJOIN._serialized_end=3257
  _TIMING._serialized_start=3259
  _TIMING._serialized_end=3306
  _OPERATORSTATS._serialized_start=3309
  _OPERATORSTATS._serialized_end=4688
  _ORDERBYDESC._serialized_start=4690
  _ORDERBYDESC._serialized_end=4775
  _MERGESORT._serialized_start=4777
  _MERGESORT._serialized_end=4853
  _UNIONALL._serialized_start=4855
  _UNIONALL._serialized_end=4891
  _BUFFER._serialized_start=4893
  _BUFFER._serialized_end=4917
  _PARTIALWINDOWFILTER._serialized_start=4919
  _PARTIALWINDOWFILTER._serialized_end=5037
  _WINDOW._serialized_start=5039
  _WINDOW._serialized_end=5112
  _WINDOWGROUP._serialized_start=5114
  _WINDOWGROUP._serialized_end=5204
  _WINDOWCALL._serialized_start=5206
  _WINDOWCALL._serialized_end=5324
  _WINDOWSPEC._serialized_start=5327
  _WINDOWSPEC._serialized_end=5613
  _WINDOWSPEC_BOUNDARYTYPE._serialized_start=5567
  _WINDOWSPEC_BOUNDARYTYPE._serialized_end=5613
  _WINDOWBOUNDARY._serialized_start=5615
  _WINDOWBOUNDARY._serialized_end=5705
  _LATERALVIEW._serialized_start=5707
  _LATERALVIEW._serialized_end=5794
  _TABLEFUNCTIONCALL._serialized_start=5796
  _TABLEFUNCTIONCALL._serialized_end=5897
  _SPOOL._serialized_start=5899
  _SPOOL._serialized_end=5924
  _CALC._serialized_start=5927
  _CALC._serialized_end=6178
  _LOGICALCALC._serialized_start=6181
  _LOGICALCALC._serialized_end=6373
  _EXPAND._serialized_start=6375
  _EXPAND._serialized_end=6432
  _GROUPINGKEYSET._serialized_start=6434
  _GROUPINGKEYSET._serialized_end=6485
  _GROUPING._serialized_start=6488
  _GROUPING._serialized_end=6722
  _CALCSTATS._serialized_start=6724
  _CALCSTATS._serialized_end=6735
  _HASHTABLESTATS._serialized_start=6738
  _HASHTABLESTATS._serialized_end=6880
  _HASHJOINSTATS._serialized_start=6883
  _HASHJOINSTATS._serialized_end=7504
  _MERGEJOINSTATS._serialized_start=7506
  _MERGEJOINSTATS._serialized_end=7522
  _HASHAGGREGATESTATS._serialized_start=7525
  _HASHAGGREGATESTATS._serialized_end=7884
  _MERGEAGGREGATESTATS._serialized_start=7887
  _MERGEAGGREGATESTATS._serialized_end=8045
  _LOCALSORTSTATS._serialized_start=8048
  _LOCALSORTSTATS._serialized_end=8431
  _MERGESORTSTATS._serialized_start=8433
  _MERGESORTSTATS._serialized_end=8449
  _UNIONALLSTATS._serialized_start=8451
  _UNIONALLSTATS._serialized_end=8466
  _VALUESSTATS._serialized_start=8468
  _VALUESSTATS._serialized_end=8481
  _BUFFERSTATS._serialized_start=8483
  _BUFFERSTATS._serialized_end=8539
  _WINDOWSTATS._serialized_start=8541
  _WINDOWSTATS._serialized_end=8554
  _PARTIALWINDOWSTATS._serialized_start=8556
  _PARTIALWINDOWSTATS._serialized_end=8576
  _EXPANDSTATS._serialized_start=8578
  _EXPANDSTATS._serialized_end=8591
  _LATERALVIEWSTATS._serialized_start=8593
  _LATERALVIEWSTATS._serialized_end=8611
  _EXCHANGESINKSTATS._serialized_start=8614
  _EXCHANGESINKSTATS._serialized_end=9027
  _EXCHANGESOURCESTATS._serialized_start=9030
  _EXCHANGESOURCESTATS._serialized_end=9344
  _TABLE._serialized_start=9346
  _TABLE._serialized_end=9429
  _VALUES._serialized_start=9431
  _VALUES._serialized_end=9538
  _SORTKEYDESC._serialized_start=9540
  _SORTKEYDESC._serialized_end=9597
  _TABLESCAN._serialized_start=9600
  _TABLESCAN._serialized_end=10102
  _TABLESCAN_PROPSENTRY._serialized_start=10058
  _TABLESCAN_PROPSENTRY._serialized_end=10102
  _TABLESCANSTATS._serialized_start=10104
  _TABLESCANSTATS._serialized_end=10167
  _TVRTABLEPROPERTY._serialized_start=10170
  _TVRTABLEPROPERTY._serialized_end=10323
  _SHUFFLETYPE._serialized_start=10326
  _SHUFFLETYPE._serialized_end=10521
  _SHUFFLETYPE_TYPE._serialized_start=10384
  _SHUFFLETYPE_TYPE._serialized_end=10521
  _TABLESINK._serialized_start=10524
  _TABLESINK._serialized_end=10866
  _SHUFFLEWRITE._serialized_start=10869
  _SHUFFLEWRITE._serialized_end=11120
  _SHUFFLEREAD._serialized_start=11123
  _SHUFFLEREAD._serialized_end=11313
  _LOCALEXCHANGE._serialized_start=11316
  _LOCALEXCHANGE._serialized_end=11506
  _LOCALSORT._serialized_start=11508
  _LOCALSORT._serialized_end=11589
  _LOGICALSORT._serialized_start=11592
  _LOGICALSORT._serialized_end=11782
  _SETOPERATOR._serialized_start=11784
  _SETOPERATOR._serialized_end=11871
  _DATAINPUTSTATS._serialized_start=11874
  _DATAINPUTSTATS._serialized_end=12094
  _FILERANGESINPUTSTATS._serialized_start=12096
  _FILERANGESINPUTSTATS._serialized_end=12170
  _FILEINPUTSTATS._serialized_start=12173
  _FILEINPUTSTATS._serialized_end=12844
  _FILEIOINPUTSTATS._serialized_start=12847
  _FILEIOINPUTSTATS._serialized_end=13032
  _FILEIOOUTPUTSTATS._serialized_start=13034
  _FILEIOOUTPUTSTATS._serialized_end=13078
  _PREFETCHSTATS._serialized_start=13081
  _PREFETCHSTATS._serialized_end=13275
  _FILEINPUTSTREAMSTATS._serialized_start=13278
  _FILEINPUTSTREAMSTATS._serialized_end=13457
  _CACHEFILEINPUTSTREAMSTATS._serialized_start=13460
  _CACHEFILEINPUTSTREAMSTATS._serialized_end=13719
  _TEXTINPUTSTATS._serialized_start=13721
  _TEXTINPUTSTATS._serialized_end=13807
  _MEMORYTABLEINPUTSTATS._serialized_start=13809
  _MEMORYTABLEINPUTSTATS._serialized_end=13853
  _PARQUETINPUTSTATS._serialized_start=13856
  _PARQUETINPUTSTATS._serialized_end=14123
  _ORCINPUTSTATS._serialized_start=14125
  _ORCINPUTSTATS._serialized_end=14161
  _CSVINPUTSTATS._serialized_start=14163
  _CSVINPUTSTATS._serialized_end=14178
  _DUMMYINPUTSTATS._serialized_start=14180
  _DUMMYINPUTSTATS._serialized_end=14218
  _AVROINPUTSTATS._serialized_start=14220
  _AVROINPUTSTATS._serialized_end=14236
  _ARROWINPUTSTATS._serialized_start=14238
  _ARROWINPUTSTATS._serialized_end=14255
  _DATAOUTPUTSTATS._serialized_start=14258
  _DATAOUTPUTSTATS._serialized_end=14469
  _FILEOUTPUTSTATS._serialized_start=14472
  _FILEOUTPUTSTATS._serialized_end=14949
  _MULTIPLEFILEOUTPUTSTATS._serialized_start=14951
  _MULTIPLEFILEOUTPUTSTATS._serialized_end=15030
  _TEXTOUTPUTSTATS._serialized_start=15032
  _TEXTOUTPUTSTATS._serialized_end=15049
  _PARQUETOUTPUTSTATS._serialized_start=15051
  _PARQUETOUTPUTSTATS._serialized_end=15150
  _ORCOUTPUTSTATS._serialized_start=15152
  _ORCOUTPUTSTATS._serialized_end=15168
  _AVROOUTPUTSTATS._serialized_start=15170
  _AVROOUTPUTSTATS._serialized_end=15187
  _ARROWOUTPUTSTATS._serialized_start=15189
  _ARROWOUTPUTSTATS._serialized_end=15207
  _TABLESINKSTATS._serialized_start=15209
  _TABLESINKSTATS._serialized_end=15274
  _SPILLSTATS._serialized_start=15276
  _SPILLSTATS._serialized_end=15371
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data_type.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x64\x61ta_type.proto\x12\x08\x63z.proto\"\x1e\n\x0c\x43harTypeInfo\x12\x0e\n\x06length\x18\x01 \x01(\x04\"!\n\x0fVarCharTypeInfo\x12\x0e\n\x06length\x18\x01 \x01(\x04\"3\n\x0f\x44\x65\x63imalTypeInfo\x12\x11\n\tprecision\x18\x01 \x01(\x04\x12\r\n\x05scale\x18\x02 \x01(\x04\"\x8b\x01\n\x13IntervalDayTimeInfo\x12$\n\x04\x66rom\x18\x01 \x01(\x0e\x32\x16.cz.proto.IntervalUnit\x12\"\n\x02to\x18\x02 \x01(\x0e\x32\x16.cz.proto.IntervalUnit\x12*\n\tprecision\x18\x03 \x01(\x0e\x32\x17.cz.proto.TimestampUnit\"a\n\x15IntervalYearMonthInfo\x12$\n\x04\x66rom\x18\x01 \x01(\x0e\x32\x16.cz.proto.IntervalUnit\x12\"\n\x02to\x18\x02 \x01(\x0e\x32\x16.cz.proto.IntervalUnit\"8\n\rArrayTypeInfo\x12\'\n\x0b\x65lementType\x18\x01 \x01(\x0b\x32\x12.cz.proto.DataType\"Y\n\x0bMapTypeInfo\x12#\n\x07keyType\x18\x01 \x01(\x0b\x32\x12.cz.proto.DataType\x12%\n\tvalueType\x18\x02 \x01(\x0b\x32\x12.cz.proto.DataType\"y\n\x0eStructTypeInfo\x12.\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x1e.cz.proto.StructTypeInfo.Field\x1a\x37\n\x05\x46ield\x12\x0c\n\x04name\x18\x01 \x01(\t\x12 \n\x04type\x18\x02 \x01(\x0b\x32\x12.cz.proto.DataType\"]\n\x10\x46unctionTypeInfo\x12 \n\x04\x61rgs\x18\x01 \x03(\x0b\x32\x12.cz.proto.DataType\x12\'\n\x0breturn_type\x18\x02 \x01(\x0b\x32\x12.cz.proto.DataType\"8\n\rTimestampInfo\x12\'\n\x06tsUnit\x18\x01 \x01(\x0e\x32\x17.cz.proto.TimestampUnit\"\x82\x05\n\x08\x44\x61taType\x12,\n\x08\x63\x61tegory\x18\x01 \x01(\x0e\x32\x1a.cz.proto.DataTypeCategory\x12\x10\n\x08nullable\x18\x02 \x01(\x08\x12\x10\n\x08\x66ield_id\x18\x0b \x01(\r\x12.\n\x0c\x63harTypeInfo\x18\x03 \x01(\x0b\x32\x16.cz.proto.CharTypeInfoH\x00\x12\x34\n\x0fvarCharTypeInfo\x18\x04 \x01(\x0b\x32\x19.cz.proto.VarCharTypeInfoH\x00\x12\x34\n\x0f\x64\x65\x63imalTypeInfo\x18\x05 \x01(\x0b\x32\x19.cz.proto.DecimalTypeInfoH\x00\x12\x30\n\rarrayTypeInfo\x18\x06 \x01(\x0b\x32\x17.cz.proto.ArrayTypeInfoH\x00\x12,\n\x0bmapTypeInfo\x18\x07 \x01(\x0b\x32\x15.cz.proto.MapTypeInfoH\x00\x12\x32\n\x0estructTypeInfo\x18\x08 \x01(\x0b\x32\x18.cz.proto.StructTypeInfoH\x00\x12?\n\x16interval_day_time_info\x18\t \x01(\x0b\x32\x1d.cz.proto.IntervalDayTimeInfoH\x00\x12\x43\n\x18interval_year_month_info\x18\n \x01(\x0b\x32\x1f.cz.proto.IntervalYearMonthInfoH\x00\x12\x31\n\x0etimestamp_info\x18\x0c \x01(\x0b\x32\x17.cz.proto.TimestampInfoH\x00\x12\x33\n\rfunction_info\x18\r \x01(\x0b\x32\x1a.cz.proto.FunctionTypeInfoH\x00\x42\x06\n\x04info*\xc1\x02\n\x10\x44\x61taTypeCategory\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04INT8\x10\x01\x12\t\n\x05INT16\x10\x02\x12\t\n\x05INT32\x10\x03\x12\t\n\x05INT64\x10\x04\x12\x0b\n\x07\x46LOAT32\x10\x05\x12\x0b\n\x07\x46LOAT64\x10\x06\x12\x0b\n\x07\x44\x45\x43IMAL\x10\x07\x12\x0b\n\x07\x42OOLEAN\x10\x08\x12\x08\n\x04\x43HAR\x10\t\x12\x0b\n\x07VARCHAR\x10\n\x12\n\n\x06STRING\x10\x0b\x12\n\n\x06\x42INARY\x10\x0c\x12\x08\n\x04\x44\x41TE\x10\r\x12\x11\n\rTIMESTAMP_LTZ\x10\x0e\x12\x17\n\x13INTERVAL_YEAR_MONTH\x10\x0f\x12\x15\n\x11INTERVAL_DAY_TIME\x10\x10\x12\n\n\x06\x42ITMAP\x10\x32\x12\t\n\x05\x41RRAY\x10\x64\x12\x07\n\x03MAP\x10\x65\x12\n\n\x06STRUCT\x10\x66\x12\x11\n\rFUNCTION_TYPE\x10g\x12\t\n\x04VOID\x10\xc8\x01*f\n\x0cIntervalUnit\x12\x16\n\x12NONE_INTERVAL_UNIT\x10\x00\x12\x08\n\x04YEAR\x10\x01\x12\t\n\x05MONTH\x10\x02\x12\x07\n\x03\x44\x41Y\x10\x03\x12\x08\n\x04HOUR\x10\x04\x12\n\n\x06MINUTE\x10\x05\x12\n\n\x06SECOND\x10\x06*Q\n\rTimestampUnit\x12\x0b\n\x07SECONDS\x10\x00\x12\x10\n\x0cMILLISECONDS\x10\x03\x12\x10\n\x0cMICROSECONDS\x10\x06\x12\x0f\n\x0bNANOSECONDS\x10\tB\x0c\n\x08\x63z.protoP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'data_type_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\010cz.protoP\001'
  _DATATYPECATEGORY._serialized_start=1461
  _DATATYPECATEGORY._serialized_end=1782
  _INTERVALUNIT._serialized_start=1784
  _INTERVALUNIT._serialized_end=1886
  _TIMESTAMPUNIT._serialized_start=1888
  _TIMESTAMPUNIT._serialized_end=1969
  _CHARTYPEINFO._serialized_start=29
  _CHARTYPEINFO._serialized_end=59
  _VARCHARTYPEINFO._serialized_start=61
  _VARCHARTYPEINFO._serialized_end=94
  _DECIMALTYPEINFO._serialized_start=96
  _DECIMALTYPEINFO._serialized_end=147
  _INTERVALDAYTIMEINFO._serialized_start=150
  _INTERVALDAYTIMEINFO._serialized_end=289
  _INTERVALYEARMONTHINFO._serialized_start=291
  _INTERVALYEARMONTHINFO._serialized_end=388
  _ARRAYTYPEINFO._serialized_start=390
  _ARRAYTYPEINFO._serialized_end=446
  _MAPTYPEINFO._serialized_start=448
  _MAPTYPEINFO._serialized_end=537
  _STRUCTTYPEINFO._serialized_start=539
  _STRUCTTYPEINFO._serialized_end=660
  _STRUCTTYPEINFO_FIELD._serialized_start=605
  _STRUCTTYPEINFO_FIELD._serialized_end=660
  _FUNCTIONTYPEINFO._serialized_start=662
  _FUNCTIONTYPEINFO._serialized_end=755
  _TIMESTAMPINFO._serialized_start=757
  _TIMESTAMPINFO._serialized_end=813
  _DATATYPE._serialized_start=816
  _DATATYPE._serialized_end=1458
# @@protoc_insertion_point(module_scope)
